// Implementation of ConstructionSiteWindow
// 9/25/2008
// Michael L. Gleicher

#include "ConstructionSiteWindow.H"

// include FlTk stuff - note: we need to turn off warnings since their
// code is sloppy
#pragma warning(push)
#pragma warning(disable:4312)
#pragma warning(disable:4311)
#include <Fl/Fl.H>
#include <FL/Fl_Double_Window.H>
#include <FL/Fl_Group.H>
#include <FL/Fl_Browser.H>
#include <FL/Fl_Value_Slider.H>
#include <FL/Fl_Button.H>
#pragma warning(pop)

#include "SiteDisplay.H"
#include "Machine.H"

// callback functions for widgets
void selectCB(Fl_Browser* b, ConstructionSiteWindow* csw)
{
	int s = b->value();
	if (s) {
		Machine *m = static_cast<Machine*>(b->data(s));
		printf("Selected Machine %d '%s'\n",s,m->name.c_str());
		csw->select(m);
	} else
		csw->select(0);
}
void sliderCB(Fl_Valuator* v, ConstructionSiteWindow* csw)
{
	if (csw->selected) {
		// we don't know which slider was pulled, so we update all
		size_t mp = std::min(csw->sliders.size(),csw->selected->parameters.size());
		for(size_t i=0; i<mp; i++) 
			csw->selected->parameters[i] = static_cast<float>(csw->sliders[i]->value());
		csw->site->damage(1);
	}
}
void cameraCB(Fl_Button* b, ConstructionSiteWindow* csw)
{	
	csw->site->damage(1);
}

void rockCB(Fl_Button* b, ConstructionSiteWindow* csw)
{	
	if (csw->selected) {
		float x,y,z;
		if (csw->selected->endEffectorPos(x,y,z)) {
			csw->rocks.push_back(Rock(x,y,z));
			csw->site->damage(1);
		}
	}
}
void crockCB(Fl_Button* b, ConstructionSiteWindow* csw)
{	
	csw->rocks.clear();
	csw->site->damage(1);
}

void resetCB(Fl_Button* b, ConstructionSiteWindow* csw)
{	
	csw->site->resetArcball();
	csw->site->damage(1);
}

//
ConstructionSiteWindow::ConstructionSiteWindow(const int x, const int y) 
	: Fl_Double_Window(x,y,800,600,"Construction Site"),
		selected(0)
{
	// create all of the widgets
	// note that they get placed in a group to make resizing work better
	widgets = new Fl_Group(630, 0, 170, 590);
	// widgets is now the group that all new widgets get put into
	// note: this code was derived from auto-generated code from Fluid
	{
		// Browser for Machines
		machinesBrowser = new Fl_Browser(635, 5, 160, 105, "select machine");
        machinesBrowser->type(2);
        Fl_Group::current()->resizable(machinesBrowser);
		machinesBrowser->callback((Fl_Callback*)selectCB,this);

		// buttons for rocks
		rockButton = new Fl_Button(635,170,75,20,"Add Rock");
		cRockButton = new Fl_Button(715,170,80,20,"Clear Rocks");
		rockButton->callback((Fl_Callback*)rockCB,this);
		cRockButton->callback((Fl_Callback*)crockCB,this);

		// Sliders - note that we're just turning everything into a Valuator
		// (parent class of slider), since we might change the type later
		for(int i=0; i<10; i++) {
			Fl_Value_Slider* s = new Fl_Value_Slider(635, 200 + 40*i, 160, 20, "unused");
			sliders.push_back(s);
			s->type(1);
			s->minimum(-100);
			s->maximum(100);
			s->step(1);
			s->deactivate();
			s->callback((Fl_Callback*)sliderCB,this);
		}

		// Buttons for camera mode
		worldCam = new Fl_Button(635, 132, 50, 20, "world");
        worldCam->type(102);
        worldCam->value(1);
        worldCam->selection_color((Fl_Color)3);
		worldCam->callback((Fl_Callback*)cameraCB,this);
 
		machineCam = new Fl_Button(685, 132, 60, 20, "machine");
        machineCam->type(102);
        machineCam->selection_color((Fl_Color)3);
		machineCam->callback((Fl_Callback*)cameraCB,this);

		driverCam = new Fl_Button(745, 132, 50, 20, "driver");
        driverCam->type(102);
        driverCam->selection_color((Fl_Color)3);
		driverCam->callback((Fl_Callback*)cameraCB,this);

		Fl_Button* rb = new Fl_Button(635, 152, 50, 18, "reset");
		rb->callback((Fl_Callback*)resetCB,this);

	};
	widgets->end(); // everything else goes into something other than the widgets group

	site = new SiteDisplay(5,5,625,590);
	site->csw = this;

	end(); // stop putting widgets into this window
}

// Destructor - note that FlTk takes care of most of the work of dealing
// with the widgets
ConstructionSiteWindow::~ConstructionSiteWindow()
{
}

//
void ConstructionSiteWindow::add(Machine* m)
{
	machines.push_back(m);
	machinesBrowser->add(m->name.c_str(),m);
	m->csw = this;
}

//
void ConstructionSiteWindow::select(Machine* m)
{
	if (m) {
		size_t mp = std::min(sliders.size(),m->parameters.size());
		size_t i;
		for(i=0; i<mp; i++) {
			sliders[i]->activate();
			sliders[i]->range(m->paramRange[i].first,m->paramRange[i].second);
			sliders[i]->label(m->paramNames[i].c_str());
			sliders[i]->value(m->parameters[i]);
		}
		for(; i<10; i++) {
			sliders[i]->label("unused");
			sliders[i]->deactivate();
		}
		selected = m;
	} else {
		for(size_t i=0; i<10; i++)
			sliders[i]->deactivate();
		selected = 0;
	}
	site->damage(1);
}

// CVS Header - if you don't know what this is, don't worry about it
// $Header: /u/c/s/cs559-1/private/CVS/ConstructionSite/ConstructionSite/ConstructionSiteWindow.cpp,v 1.7 2008/10/08 16:25:33 gleicher Exp $
