#pragma once
// ConstructionSiteWindow.H
// begun 9/25/2008 - Michael Gleicher
//
// Header file that describes the Window/World of the construction site
// The main thing here is the definition of the class that is the 
// Window (and container for other objects)
//
// Note that there are two windows:
// The "outer" window that contains the widgets for control (including the inner window)
// The "inner" window that is the GL context into which the world is drawn
//
// ConstructionSiteWindow is the outer window
// SiteDisplay is the inner window, and is defined in another file (but we have a 
//		"forward" reference here:
class SiteDisplay;

// standard useful stuff
#include <vector>

// since this is an FLTk Window, we need to include FlTk Headers
// the pragmas turn off the warnings from FlTk - our code should have no warnings,
// but their code doesn't meet that standard
#pragma warning(push)
#pragma warning(disable:4312)
#pragma warning(disable:4311)
#include <Fl/Fl_Double_Window.h>
#pragma warning(pop)

// rather than including all of the widget types, we just declare the types
// here - if you want to use them, be sure to include them in the C++ file
class FL_Slider;
class Fl_Group;
class Fl_Browser;
class Fl_Valuator;
class Fl_Button;

// the actual objects that we'll put into the world
class Machine;

// a simple structure for the positions of the rocks
struct Rock {
	float x,y,z;
	inline Rock(float _x, float _y, float _z) : x(_x), y(_y), z(_z) {};
};

// the class for the user interface - 
// note: i have erred on the side of just making everything public
class ConstructionSiteWindow : public Fl_Double_Window {
public:
	ConstructionSiteWindow(const int x=50, const int y=50);
	~ConstructionSiteWindow();

	// the most import UI element
	SiteDisplay* site;

	// the widgets for the UI of this thing
	Fl_Group *widgets;
	Fl_Browser *machinesBrowser;
	std::vector<Fl_Valuator*> sliders;
	Fl_Button *worldCam;
	Fl_Button *machineCam;
	Fl_Button *driverCam;
	Fl_Button *rockButton;
	Fl_Button *cRockButton;

	// add a new machine to the world
	void add(Machine*);

	// keep track of what's in the world - only add to this with the add method
	std::vector<Machine*> machines;

	// things for controlling the selection
	Machine* selected;		// becomes the same as what's shown
	void select(Machine*);	// set this to be the selected machine

	// rocks
	std::vector<Rock> rocks;
};

// CVS Header - if you don't know what this is, don't worry about it
// $Header: /u/c/s/cs559-1/private/CVS/ConstructionSite/ConstructionSite/ConstructionSiteWindow.h,v 1.4 2008/09/29 19:16:11 gleicher Exp $