#pragma once

// ConstructionSite - CS559 example program
// Mike Gleicher, 9/25/2008
// Machine.H - Definition of the base class of the "machines" - 
//    the actual objects that will drive around in the world

// The main part of the assignment is to provide subclasses of this class.
// The base class is "purely virtual" (you must override some of its methods)
// You also must provide a constructor.

// You shouldn't need to modify this file, but you will need to include
// it and make subclasses from it

#include <vector>
#include <string>
#include <utility>		// has std::pair

// note: the machine is aware of the world/window its in (unfortunately)
// the alternative would be to have the world/window be a global 
// variable. in a small project like this one, that wouldn't be so bad.
// but for habit, I do it the other way.
class ConstructionSiteWindow;


//////////////////////////////////////////////////
class Machine
{
public:
	// the constructor - note that your constructor (for the subclass)
	// must call this with the appropriate arguments
	Machine(const char* name, const int nParams);

	// the main work of the class is in the following methods 
	// (as well as the constructor that sets up all the variables)
	
	// these are the methods that your subclass should override
	// the purely virtual ones MUST be overridden

	// have the object draw itself - you can assume that this only
	// happens when there is an actual drawing context, and the camera/lighting
	// is set up appropriately
	virtual void draw() = 0;

	// have the object tell us where its end effector is (in order to place a rock)
	// if the object doesn't have an end-effector (or can't figure out where it is),
	// this method should return false.
	// otherwise, it should return true and set x,y,z accordingly (they are passed
	// by reference)
	// note: the base class implementation does provide a "null" version (saying that
	// this class cannot do it)
	virtual bool endEffectorPos(float& x, float& y, float& z);

	// these methods have the machine set the camera
	// when setting the camera, the machine is responsible for clearing out
	// the matrices (both Projection and ModelView) and setting them appropriately
	// we pass the aspect ration (width/height) of the window since that's needed
	// to set up the transform

	// this routine sets the camera to look at the machine. it is required for all
	// new subclasses
	virtual void lookAtMe(float) = 0;

	// this routine sets the camera to look "from" the machine, the direction the
	// driver would see it
	// if the machine cannot do this (for example, it doesn't have a driver), then
	// it should return false, and another camera will take over
	virtual bool driversView(float);

	// the parameters - each vector should have the same length
	// remember that parameters 0-2 are reserved for X,Z,THETA 
	//	(remember, Y is up, so its always 0, since things are on the ground)
	// everything else, is up to the child class ...
	// information about the parameters is held in variables below
	std::vector<float> parameters;

	// things that we need to remember about each thing
	// even though these probably don't change, we still store
	// them in variables for simplicity
	
	// name of this machine
	std::string name;
	// here we store info about the parameters
	std::vector<std::string> paramNames;
	std::vector<std::pair<float,float> > paramRange;	// min, max for each one

	// keep track of the window (see comment above)
	ConstructionSiteWindow* csw;
};

// CVS Header - if you don't know what this is, don't worry about it
// $Header: /p/course/cs559-gleicher/private/CVS/ConstructionSite/ConstructionSite/Machine.h,v 1.3 2008/09/29 18:50:46 gleicher Exp $
