#include "SiteDisplay.H"
#include "ConstructionSiteWindow.H"
#include "Machine.H"
#include "Utilities/3DUtils.h"

#include <windows.h>
#include "GL/gl.h"

#pragma warning(push)
#pragma warning(disable:4312)
#pragma warning(disable:4311)
#include <FL/Fl_Button.H>
#pragma warning(pop)

using std::vector;

SiteDisplay::SiteDisplay(int x, int y, int w, int h, const char* l) : Fl_Gl_Window(x,y,w,h,l)
{
	mode( FL_RGB|FL_ALPHA|FL_DOUBLE | FL_STENCIL );

	resetArcball();
}

void SiteDisplay::resetArcball()
{
	// set up the camera to look at the world
	// these parameters might seem magical, and they kindof are
	// a little trial and error goes a long way
	arcball.reset();
	arcball.setup(this,40,250);
	arcball.spin(.2f,.4f,0);
}

int SiteDisplay::handle(int event)
{
	// see if the ArcBall will handle the event - if it does, then we're done
	if (!csw->machineCam->value() && !csw->driverCam->value())
		if (arcball.handle(event)) return 1;

	return Fl_Gl_Window::handle(event);
}

void SiteDisplay::draw()
{
	// clear the window, be sure to clear the Z-Buffer too
	glClearColor(0,0,.3f,0);		// background should be blue
	// we need to clear out the stencil buffer since we'll use
	// it for shadows
	glClearStencil(0);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);

	// enable the lighting
	glEnable(GL_COLOR_MATERIAL);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glEnable(GL_LIGHT1);
	glEnable(GL_LIGHT2);
	glEnable(GL_DEPTH);

	// set up the lighting
	GLfloat lightPosition[] = {50, 200.0, 50, 1.0};
	GLfloat lightPosition2[] = {-100.0, 10.0, -150.0, 1.0};
	GLfloat lightPosition3[] = {0, -200, 0, 1.0};
	GLfloat violetLight[] = {0.5f, 0.1f, .5f, 1.0};
	GLfloat yellowishLight[] = {1, 1, 1, 1.0};
	GLfloat blueLight[] = {.1f,.1f,.3f,1.0};

	// Blayne prefers GL_DIFFUSE
    glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);

	// set up the projection
	// taken from CubeIt2
	glMatrixMode(GL_PROJECTION);
	glViewport(0,0,w(),h());

	// compute the aspect ratio (we'll need it)
	float aspect = static_cast<float>(w()) / static_cast<float>(h());

	// depending on the camera, do the right thing
	if (csw->driverCam->value()) {
		// driver cam only works if there's a selected machine, and it can set camera
		bool didDriver = false;
		if (csw->selected) 
			didDriver = csw->selected->driversView(aspect);
		if (!didDriver) {
			// couldn't use drivercam, so use the normal one
			arcball.setProjection(true);
			// TODO: we might want to make it switch the buttons as well
		}
	} else if (csw->machineCam->value()) {
		if (csw->selected)
			csw->selected->lookAtMe(aspect);
		else
			arcball.setProjection(true);
	} else { // world camera mode
		arcball.setProjection(true);
	}

	glLightfv(GL_LIGHT0, GL_POSITION, lightPosition);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, yellowishLight);

	glLightfv(GL_LIGHT1, GL_POSITION, lightPosition2);
	glLightfv(GL_LIGHT1, GL_DIFFUSE, yellowishLight);

	glLightfv(GL_LIGHT2, GL_POSITION, lightPosition3);
	glLightfv(GL_LIGHT2, GL_DIFFUSE, blueLight);

	// now draw the ground plane
	setupFloor();
	drawFloor(200,10);

	// draw the rocks
	glColor3ub(140,98,57);		// brown
	for(vector<Rock>::iterator i = csw->rocks.begin(); i != csw->rocks.end(); ++i)
		drawCube(i->x,i->y,i->z,2);

	// now draw the machines
	for(vector<Machine*>::iterator i=csw->machines.begin(); i != csw->machines.end(); ++i)
		(*i)->draw();	
}

// CVS Header - if you don't know what this is, don't worry about it
// $Header: /u/c/s/cs559-1/private/CVS/ConstructionSite/ConstructionSite/SiteDisplay.cpp,v 1.11 2008/10/08 17:12:33 bfield Exp $
